/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Mathias Preiner, Aina Niemetz
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__DRIVER_H
#define CVC5__OPTIONS__DRIVER_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

namespace driver::longName {
// clang-format off
  static constexpr const char* showCopyright = "copyright";
    static constexpr const char* dumpDifficulty = "dump-difficulty";
    static constexpr const char* dumpInstantiations = "dump-instantiations";
    static constexpr const char* dumpInstantiationsDebug = "dump-instantiations-debug";
    static constexpr const char* dumpModels = "dump-models";
    static constexpr const char* dumpProofs = "dump-proofs";
    static constexpr const char* dumpUnsatCores = "dump-unsat-cores";
    static constexpr const char* dumpUnsatCoresLemmas = "dump-unsat-cores-lemmas";
    static constexpr const char* earlyExit = "early-exit";
    static constexpr const char* filename = "filename";
    static constexpr const char* forceNoLimitCpuWhileDump = "force-no-limit-cpu-while-dump";
    static constexpr const char* help = "help";
    static constexpr const char* helpRegular = "help-regular";
    static constexpr const char* interactive = "interactive";
    static constexpr const char* portfolioJobs = "portfolio-jobs";
    static constexpr const char* printSuccess = "print-success";
    static constexpr const char* seed = "seed";
    static constexpr const char* segvSpin = "segv-spin";
    static constexpr const char* showConfiguration = "show-config";
    static constexpr const char* showTraceTags = "show-trace-tags";
    static constexpr const char* usePortfolio = "use-portfolio";
    static constexpr const char* showVersion = "version";
// clang-format on
}

// clang-format off

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderDRIVER
{
// clang-format off
  bool showCopyright = false;
  bool showCopyrightWasSetByUser = false;
  bool dumpDifficulty = false;
  bool dumpDifficultyWasSetByUser = false;
  bool dumpInstantiations = false;
  bool dumpInstantiationsWasSetByUser = false;
  bool dumpInstantiationsDebug = false;
  bool dumpInstantiationsDebugWasSetByUser = false;
  bool dumpModels = false;
  bool dumpModelsWasSetByUser = false;
  bool dumpProofs = false;
  bool dumpProofsWasSetByUser = false;
  bool dumpUnsatCores = false;
  bool dumpUnsatCoresWasSetByUser = false;
  bool dumpUnsatCoresLemmas = false;
  bool dumpUnsatCoresLemmasWasSetByUser = false;
  bool earlyExit = true;
  bool earlyExitWasSetByUser = false;
  std::string filename = "";
  bool filenameWasSetByUser = false;
  bool forceNoLimitCpuWhileDump = false;
  bool forceNoLimitCpuWhileDumpWasSetByUser = false;
  bool help = false;
  bool helpWasSetByUser = false;
  bool helpRegular = false;
  bool helpRegularWasSetByUser = false;
  bool interactive = false;
  bool interactiveWasSetByUser = false;
  uint64_t portfolioJobs = 1;
  bool portfolioJobsWasSetByUser = false;
  bool printSuccess = false;
  bool printSuccessWasSetByUser = false;
  uint64_t seed = 0;
  bool seedWasSetByUser = false;
  bool segvSpin = false;
  bool segvSpinWasSetByUser = false;
  bool showConfiguration = false;
  bool showConfigurationWasSetByUser = false;
  bool showTraceTags = false;
  bool showTraceTagsWasSetByUser = false;
  bool usePortfolio = false;
  bool usePortfolioWasSetByUser = false;
  bool showVersion = false;
  bool showVersionWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__DRIVER_H */
