/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Mathias Preiner, Aina Niemetz
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2025 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__PROOF_H
#define CVC5__OPTIONS__PROOF_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

namespace proof::longName {
// clang-format off
  static constexpr const char* checkProofSteps = "check-proof-steps";
    static constexpr const char* dratBinaryFormat = "drat-binary-format";
    static constexpr const char* lfscExpandTrust = "lfsc-expand-trust";
    static constexpr const char* lfscFlatten = "lfsc-flatten";
    static constexpr const char* optResReconSize = "opt-res-reconstruction-size";
    static constexpr const char* printDotClusters = "print-dot-clusters";
    static constexpr const char* proofAletheDefineSkolems = "proof-alethe-define-skolems";
    static constexpr const char* proofAletheResPivots = "proof-alethe-res-pivots";
    static constexpr const char* proofAllowTrust = "proof-allow-trust";
    static constexpr const char* proofCheck = "proof-check";
    static constexpr const char* proofDagGlobal = "proof-dag-global";
    static constexpr const char* printDotAsDAG = "proof-dot-dag";
    static constexpr const char* proofElimSubtypes = "proof-elim-subtypes";
    static constexpr const char* proofFormatMode = "proof-format-mode";
    static constexpr const char* proofGranularityMode = "proof-granularity";
    static constexpr const char* proofPedantic = "proof-pedantic";
    static constexpr const char* proofPpMerge = "proof-pp-merge";
    static constexpr const char* proofPrintConclusion = "proof-print-conclusion";
    static constexpr const char* proofPrintReference = "proof-print-reference";
    static constexpr const char* proofPruneInput = "proof-prune-input";
    static constexpr const char* proofRewriteRconsRecLimit = "proof-rewrite-rcons-rec-limit";
    static constexpr const char* proofRewriteRconsStepLimit = "proof-rewrite-rcons-step-limit";
    static constexpr const char* propProofMode = "prop-proof-mode";
    static constexpr const char* satProofMinDimacs = "sat-proof-min-dimacs";
// clang-format on
}

// clang-format off
enum class ProofCheckMode
{
  EAGER, EAGER_SIMPLE, LAZY, NONE,
  __MAX_VALUE = NONE
};
std::ostream& operator<<(std::ostream& os, ProofCheckMode mode);
ProofCheckMode stringToProofCheckMode(const std::string& optarg);

enum class ProofFormatMode
{
  NONE, DOT, LFSC, ALETHE, CPC,
  __MAX_VALUE = CPC
};
std::ostream& operator<<(std::ostream& os, ProofFormatMode mode);
ProofFormatMode stringToProofFormatMode(const std::string& optarg);

enum class ProofGranularityMode
{
  MACRO, REWRITE, THEORY_REWRITE, DSL_REWRITE, DSL_REWRITE_STRICT,
  __MAX_VALUE = DSL_REWRITE_STRICT
};
std::ostream& operator<<(std::ostream& os, ProofGranularityMode mode);
ProofGranularityMode stringToProofGranularityMode(const std::string& optarg);

enum class PropProofMode
{
  PROOF, SAT_EXTERNAL_PROVE,
  __MAX_VALUE = SAT_EXTERNAL_PROVE
};
std::ostream& operator<<(std::ostream& os, PropProofMode mode);
PropProofMode stringToPropProofMode(const std::string& optarg);

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderPROOF
{
// clang-format off
  bool checkProofSteps = false;
  bool checkProofStepsWasSetByUser = false;
  bool dratBinaryFormat = false;
  bool dratBinaryFormatWasSetByUser = false;
  bool lfscExpandTrust = false;
  bool lfscExpandTrustWasSetByUser = false;
  bool lfscFlatten = false;
  bool lfscFlattenWasSetByUser = false;
  bool optResReconSize = true;
  bool optResReconSizeWasSetByUser = false;
  bool printDotClusters = false;
  bool printDotClustersWasSetByUser = false;
  bool proofAletheDefineSkolems = false;
  bool proofAletheDefineSkolemsWasSetByUser = false;
  bool proofAletheResPivots = false;
  bool proofAletheResPivotsWasSetByUser = false;
  bool proofAllowTrust = true;
  bool proofAllowTrustWasSetByUser = false;
  ProofCheckMode proofCheck = ProofCheckMode::NONE;
  bool proofCheckWasSetByUser = false;
  bool proofDagGlobal = true;
  bool proofDagGlobalWasSetByUser = false;
  bool printDotAsDAG = false;
  bool printDotAsDAGWasSetByUser = false;
  bool proofElimSubtypes = false;
  bool proofElimSubtypesWasSetByUser = false;
  ProofFormatMode proofFormatMode = ProofFormatMode::CPC;
  bool proofFormatModeWasSetByUser = false;
  ProofGranularityMode proofGranularityMode = ProofGranularityMode::MACRO;
  bool proofGranularityModeWasSetByUser = false;
  uint64_t proofPedantic = 0;
  bool proofPedanticWasSetByUser = false;
  bool proofPpMerge = true;
  bool proofPpMergeWasSetByUser = false;
  bool proofPrintConclusion = false;
  bool proofPrintConclusionWasSetByUser = false;
  bool proofPrintReference = false;
  bool proofPrintReferenceWasSetByUser = false;
  bool proofPruneInput = false;
  bool proofPruneInputWasSetByUser = false;
  uint64_t proofRewriteRconsRecLimit = 5;
  bool proofRewriteRconsRecLimitWasSetByUser = false;
  uint64_t proofRewriteRconsStepLimit = 1000;
  bool proofRewriteRconsStepLimitWasSetByUser = false;
  PropProofMode propProofMode = PropProofMode::PROOF;
  bool propProofModeWasSetByUser = false;
  bool satProofMinDimacs = true;
  bool satProofMinDimacsWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__PROOF_H */
